function array3d (theta0_deg, phi0_deg)

% Compute and plot the array factor of a 3D array.
% We form the total array by adding two independently parametrised
% plane sub-arrays. The subarrays need not be on the same plane,
% but they must be parallel.
% It possible to randomize the grid.
%
% 7-Jun-2006 Jm

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

D2R = pi/180; R2D = 180/pi;

% Pointing direction

if nargin == 2
    theta0 = theta0_deg*D2R;
    phi0 = phi0_deg*D2R;
else
    theta0 = 35.0*D2R;
    phi0   = 90.0*D2R;
end

% Grid size on the disk of visibility

Nx = 100;
Ny = 100;

% Array definition

lambda = 3e8/224e6;

% Number of rows and columns

Dx = 1.4;
Dy = 1.4;

% First array

% Number of rows and columns

M1 = 12;
M2 = 2;

% Base vectors of the first array

D1 = [Dx,0,0];          
D2 = [0,2*Dy,0];
D3 = [0,0,0];

% Second array

N1 = 12;
N2 = 2;

E1 = [Dx,0,0];
E2 = [0,2*Dy,0];
E3 = [0,0,0];

E0 = [0.0*Dx,Dy,0.0]; % Offset of second arrays's origin

RandAmpXY = 0.0; % Max random offset from grid position in X and Y,
                 % in terms of wavelength.
RandAmpZ = 0.0;





%======================
% End of input section
%===========================================================

set(0,'defaultaxesfontsize',10)

M = M1*M2;
DD = [D1', D2', D3'];
EE = [E1', E2', E3'];

% Compute the 3D position vector Rm for each array element m.

Rm = zeros(3,M);

RandAmp = [RandAmpXY;RandAmpXY;RandAmpZ];
m = 0;
for m2 = 0:(M2-1)
    for m1 = 0:(M1-1)
        m = m+1;
        Rm(:,m) = DD * [m1;m2;0]  +  rand(3,1).*RandAmp;
    end
end
for n2 = 0:(N2-1)
    for n1 = 0:(N1-1)
        m = m+1;
        Rm(:,m) = EE * [n1;n2;0] + E0' + rand(3,1).*RandAmp;
    end
end


u0 = [sin(theta0)*cos(phi0), sin(theta0)*sin(phi0), cos(theta0)];


% Compute the array factor in rectangular grid on the disk of visibility.

Ux = linspace (-1,1,Nx);
Uy = linspace (-1,1,Ny);

AF = NaN .* zeros(Nx,Ny);

for n2 = 1:length(Uy)
    for n1 = 1:length(Ux)
        uu = Ux(n1)^2 + Uy(n2)^2;
        if uu < 1
            U = [Ux(n1),Uy(n2),sqrt(1-uu)];
            AF (n1,n2) = abs (sum ( exp ( i*2*pi* (U-u0) * Rm)));
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           GRAPHS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

getfig (mfilename,400,550);

HEADER = subplot ('position',[0.10 0.90 0.88 0.095]);

    D1_str = sprintf('[%1.3g,%1.3g]',D1(1),D1(2));
    D2_str = sprintf('[%1.3g,%1.3g]',D2(1),D2(2));
    if N1*N2 == 0
        fmt1 = 'D1=%s D2=%s';
        hdr{1} = sprintf ([fmt1],D1_str,D2_str);
    else
        E1_str = sprintf('[%1.3g,%1.3g]',E1(1),E1(2));
        E2_str = sprintf('[%1.3g,%1.3g]',E2(1),E2(2));
        fmt1 = 'D1=%s D2=%s E1=%s E2=%s';
        hdr{1} = sprintf ([fmt1],D1_str,D2_str,E1_str,E2_str);
    end

    fmt1 = '\\theta_0=%1.2f \\phi_0=%1.2f';
    fmt2 = 'E0=[%1.3g,%1.3g,%1.3g]';
    if N1*N2 == 0
        hdr{2} = sprintf ([fmt1],theta0*R2D,phi0*R2D);
    else
        hdr{2} = sprintf ([fmt1 '  ' fmt2],theta0*R2D,phi0*R2D,E0);
    end
    figureheader (HEADER,hdr);

PANEL1 = subplot ('position',[0.15 0.62 0.8 0.25]);
%    plot_grid (M1,M2,DD, N1,N2,EE,E0');
    plot2_grid (Rm);
    

PANEL2 = subplot('position',[0.10 0.10 0.8 0.45]);

if 1
    maxAF = max(max(AF')); maxAF = 1;
    Z = (AF'/maxAF).^1; 
    h = surf (Ux,Uy,Z);
    shading interp
    xlabel ('Ux')
    ylabel ('Uy')
    h = title ('|AF|            '); set(h,'fontw','b');
    colorbar
    set (gca,'tickdir','out')
    set (gca,'ticklen',[0.03,0.03])
    axis square
    grid off
    box on
    view (0,90)
end

if 0
    maxAF = max(max(AF')); maxAF = 1;
    h = pcolor (Ux,Uy,(AF'/maxAF)); 
    shading interp
    xlabel ('Ux')
    axis square
    set (gca,'tickdir','out')
    set (gca,'ticklen',[0.03,1])
    grid off
    colorbar
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%       subfunctions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function plot_grid (M1,M2,DD, N1,N2,EE,E0)
%--------------------------------------------

M = M1*M2;
N = N1*N2;

m = 0;
m3 = 0;
for m2 = 0:(M2-1)
    for m1 = 0:(M1-1)
        m = m+1;
        R = DD * [m1;m2;0];
        if m == 1
            h = plot3 (R(1),R(2),R(3));
        else
            h = line (R(1),R(2),R(3));
        end
        set (h,'marker','o');
        set (h,'color','b')
        set (h,'markerfacecolor','b')
    end
end

for n2 = 0:(N2-1)
    for n1 = 0:(N1-1)
        R = EE * [n1;n2;0] + E0;
        h = line (R(1), R(2), R(3));
        set (h,'marker','o');
        set (h,'color','r')
        if E0(3) ~= 0
            h = line ([R(1) R(1)],[R(2),R(2)],[0 R(3)]);
            set (h,'color',[0.6 0.6 0.6])
        else
            set (h,'markerfacecolor','r')
        end
    end
end


set (gca,'zlim',[-Inf Inf]);
grid off
box on
xlabel('X / \lambda')
ylabel('Y / \lambda')
axis equal
view (-20,50);            



function plot2_grid (Rm)
%--------------------------------------------
%
%--------------------------------------------
threedee = 0;

for m = 1:length(Rm)
    R = Rm(:,m);
    if m == 1
        h = plot3 (R(1),R(2),R(3));
    else
        h = line (R(1),R(2),R(3));
    end
    set (h,'marker','o');
    set (h,'color','b')
    set (h,'markerfacecolor','b')
    
    if R(3) > 0.01
        h = line ([R(1),R(1)],[R(2),R(2)], [0,0]);
        set (h,'marker','x');
        set (h,'color','r')
        h = line ([R(1),R(1)],[R(2),R(2)], [0,R(3)]);
        set (h,'color','r')
        threedee = 1;
    end

end

set (gca,'zlim',[-Inf Inf]);
grid off
xlabel('X / \lambda')
ylabel('Y / \lambda')
axis equal
if threedee
    box off
    view (-10,70);            
else
    box on
    view (0,90);            
end



function hh = getfig(Figurename,Width,Height)
%-------------------------------------------------------
h = findobj('Tag',Figurename);
if isempty(h)
    h = figure;
    set(h,'Tag',Figurename);
    set(h,'Numbertitle','off');
    pos = get(h,'Position');
    set(h,'Position',[pos(1), pos(2), Width, Height]);
    set(h,'PaperType','A4');
    set(h,'Name',Figurename);
    set(h,'paperpos',[1 1 4 5]);
else
    figure(h);
end

if nargout > 0
    hh = h;
end    


function figureheader (h,txt)

% FIGUREHEADER (AXHANDLE, TXT)
%
% Draw a multiline figure header.
%
% AXHANDLE is axis handle for the axis where the text is drawn.
% TXT{n} contains the n'th line of the header.   

% 24-Oct-2005 Jm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cla
set(h,  'xticklabel',[],...
        'yticklabel',[],...
        'xtick',[],...
        'ytick',[],...
        'box','off',...
        'color',[1 1 1]);
        
N = length(txt);        
n = N:-1:1;
y = (2*n-1)/(2*N);
for k = 1:N
    hh = text(0.02,y(k),txt{k});
    set(hh,...
        'Color',[0 0 0],...
        'VerticalAlignment','middle'...
        );
        set(hh,'fontsize',10);

end    

%end %figureheader

