function beamshape2d

% BEAMSHAPE2D
%
% 2D contour plot of gain across an array beam, in the plain
% perpendicular to the beam. The XY contour plot is projected
% on the sky as looked along the beam direction.
%
% We assume isotropic array element patterns (that is, we 
% compute the beam shape using the array factor only).

% 27-Oct-2005 Jm
% 16-Sep-2006 Jm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

set(0,'defaultaxesfontsize',14);
d2r = pi/180;
r2d = 180/pi;
Range = [];

%=====================  BEGIN INPUT  ===================================

%AD = create_arraydesc (50, 20, 1.5, 1.5);   % Mx, My, Dx, Dy

AD = create_arraydesc (12, 4, 1.4, 1.4);   % Mx, My, Dx, Dy

phi0 = 00*d2r;          % beam azimuth (radians)

%th0 = 35*d2r;           % beam zenith angle (radians)
%th0 = 20*d2r;           % beam zenith angle (radians)
th0 = 60*d2r;           % beam zenith angle (radians)

%Range = 360;            % Range. If not empty, the counter plots
%Range = 635;
Range = 233;
                        % are projected to the given range.
                        % If empty, we use angles.

%ED.phi = 0*d2r;
%ED.th  = 55*d2r;
%ED.gain = 10;


%====================== END INPUT ======================================

%N = 301;                % Number of points on the grid axis. Must be odd.
N = 101;

Dmax = 10*d2r;           % Ax limits (radians) will be cropped to this.

if isempty (Range)
    use_degr = 1;
else
    use_degr = 0;
end

Order = 1.0;  % Drawing area roughly limited by the n'th null-gain lines 
              % around (phi0, th0).

Levels = [0.5 0.1 0.02];  % Levels at which to draw the counter lines.

% Array length and width

Lx = AD.Mx * AD.Dx;
Ly = AD.My * AD.Dy;

[ux,uy] = sph2cart ( phi0, pi/2 - th0, 1);

% The corners n'th null-line rectangle

u{1} = [ux - Order/Lx, uy - Order/Ly];        % lower left
u{2} = [ux + Order/Lx, uy - Order/Ly];        % lower right
u{3} = [ux + Order/Lx, uy + Order/Ly];        % upper right
u{4} = [ux - Order/Lx, uy + Order/Ly];        % upper left

% The corresponding points in shperical coordinates

for k = 1:4
    [phi(k), el(k)] = cart2sph (u{k}(1), u{k}(2), sqrt (1 - u{k}(1)^2 - u{k}(2)^2));
end
th = pi/2 - el;

% Find the largest distance from the center point in spherical coordinates,
% and derive limits of the drawing rectangle from that

D1 = max (abs (phi-phi0));
D2 = max (abs (th-th0));

phi1 = phi0-D1;
phi2 = phi0+D1;
th1 = th0-D2;
th2 = th0+D2;

% In this implementation, we need to ensure that we do not get
% too near vertical.
% Arbitraryly, we will not allow to get nearer than 5 deg to vertical.

if th1 < 5*d2r
    error ('Pointing too near the vertical')
end

% Compute AF on a NxN grid, where column row has constant phi, 
% and each row has constant theta.
% We want to compute in grid that is uniform in angular space, with
% step d = dtheta. 

Nper2 = floor (N/2);

th = linspace (th1,th2,N);
d = (th(end)-th(1))/(N-1);
NN = -Nper2:Nper2;

D = NN * d;

AF = zeros (N,N);



for m = 1:length(th)
    phi = phi0 + NN * ( d /sin(th(m)) );
    for k = 1:length(phi)
        AF(m,k) = arraygain (phi(k),th(m),phi0,th0,AD,[]);
    end
end

G = AF.^2;

% Flip the array so that that positive y dir means up, 
% and positive x means to right, when looking at the sky along the beam
% direction

G = fliplr(G);
G = flipud(G);

% Vertical beamwidth

Wv = beamwidth (AD.Dx,AD.Dy,AD.Mx,AD.My,phi0,th0);
fprintf(1,'Vertical width = %1.2f degr\n',Wv*r2d);

%=======================================================================
%               GRAPHS
%=======================================================================

getfigure(mfilename,[0.25 2.5 6 6], 500);

HDR = subplot('position',[0.02,0.81,0.885,0.10]);

    if use_degr
        r2u = r2d;   % conversion factor from radians to the axis units.
        fmt1 = '%d{\\times}%d %1.1f{\\times}%1.1f';  
            fmt2 = '   \\phi_0=%1.1f\\circ  \\theta_0=%1.1f\\circ';
                fmt3 = '  W_V=%1.2f\\circ';
        s = sprintf ([fmt1 fmt2 fmt3], ...
                        AD.Mx, AD.My, AD.Dx, AD.Dy, ...
                            phi0*r2d, th0*r2d, ...
                                Wv*r2u);
        xlb = 'H offset (\circ)';
        ylb = 'V offset (\circ)';

    else
        r2u = Range;  % conversion factor from radians to the axis units.
        
        fmt1 = '%d{\\times}%d %1.1f{\\times}%1.1f';  
            fmt2 = '   \\phi_0=%1.1f\\circ \\theta_0=%1.1f\\circ';
                fmt3 = '   R = %1.1f km';
                    fmt4 = '  Wv = %1.1f km (%1.2f\\circ)';
        s = sprintf ([fmt1 fmt2 fmt3 fmt4], ...
                        AD.Mx, AD.My, AD.Dx, AD.Dy, ...
                            phi0*r2d, th0*r2d, ...
                                Range, ...
                                    Wv*r2u, Wv*r2d);
        xlb = 'H offset (km)';
        ylb = 'V offset (km)';
    end
    Dmax = Dmax*r2u; % axis cropping box size

    hdr{1} = s;
    figureheader(HDR, hdr);    

PANEL = subplot('position',[0.1500    0.1300    0.6    0.6]);

    [cc,h] = contour (D*r2u, D*r2u, G, Levels, 'b-');
    clabel(cc,h)

    %keyboard
    %pcolor (D*r2u, D*r2u, G/max(max(G)));shading('interp')

    hold on
    plot ([0,0],[0,0],'b+', [0,0],[-Wv*r2u/2 Wv*r2u/2],'r-');
    hold off

    set (h, 'linewidth', 1.5);
    set (gca,'tickdir','out');
    set (gca,'ticklength',[0.015 0.025]);
    set (gca,'layer','top');

    D1 = D(1)*r2u;
    D2 = D(end)*r2u;
    DD = max ([D1,D2]);
    if DD > Dmax
        xlim = [-Dmax,Dmax];
    else
        xlim = [-DD,DD];
    end

    if xlim(2) < 6
        ticks = -5:5;
        set (gca, 'xtick', ticks);
        set (gca, 'ytick', ticks);
    elseif xlim(2) < 11 
        ticks = -10:2:10;
        set (gca, 'xtick', ticks);
        set (gca, 'ytick', ticks);
    end

    set (gca,'xlim', xlim);
    set (gca,'ylim', xlim);
    axis square;

    title ('|AF|^2'); 
    xlabel (xlb);
    ylabel (ylb);


