function beamsteering (D)

% Beam direction as function of phasing angle in a phased array.
% 
% D is array element separation in units of wavelength.  
%
% This m-file illustrates the equivalence of all the
% "grating lobes" and the "main lobe". 
% The full polar angle [-pi pi] can always be covered optimally, 
% by _some_ of the grating lobes. If we have large D ( = d/lambda),
% there are more of these lobes, so the measurement ambiguity is
% larger. The ambiguity can be reduced by using directional
% elements. The grating lobes kind of correspond to Nyqist zones,
% and the main beam to the "baseband" zone.
%
% The situation is quite analogous to the sampling of signal s(t).
% Here the element directivity corresponds to a (band-bass) filter,
% that picks only some particular Nyqist zone.
%
% Unless we know apriori in which "Nyqis zone" (solid angle zone) the
% target is, the target direction can only be determined modulo to the
% the cell structure. 
%
% The beam directions ("grating" and "base") theta are found from
%
%   sin(theta) = n/D + delta/(2piD),            (*)
%
% with n = 0, +-1, ... +- nmax (nmax=ceil(D)) and the phasing step delta
% restricted to [-pi,pi[ with n = 0, and to some subinterval near nmax,
% such that -1 < n/D + delta/2piD < 1 holds.  
%
% The representation (*) corresponds to dividing the angular theta space
% to the "Nyqist zones".
% 
% But we can equally well compute the beam directions from
%
%   sin(theta) = delta/(2piD),                  (**)
%
% with delta now restrected not to [-pi pi], but to
%
%           -D*2pi < delta < D*2pi
%
% which is kind of simper condition for delta. Clearly, what ever the
% theta, there is always a solution delta for that angle.
%
% Both representations (*) and (**) are fully equivalent, signifying 
% the fact that there is _no_ way to say from which beam we get the signal.

% 20-Oct-2005 Jm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin ~= 1
    error ('Bad number of arguments')
end

set(0,'defaultaxesfontsize',14);

% Element separation in terms of wavelength

d2r = pi/180;
r2d = 180/pi;




%=====================================================================
% (A) First draw the beam direction according to (**), allowing delta
% to vary over more than 2pi.
%=====================================================================

getfigure('beamsteering (1)',[0.25 2.5 7 4], 500);

x1 = min ([-D -1.5]);
x2 = max ([D +1.5]);

delta1 = -2*pi*D;
delta2 = 2*pi*D;

delta = delta1:0.001:delta2;

y = asin ( delta/(2*pi*D) );

% mark out the "base zone" by painting a colored band

y1 = asin (-0.5/D) * r2d;
y2 = asin (0.5/D) * r2d;
h = area ( [x1 x2], [y2 y2], y1 );
    set (h, 'facecolor', [0.85 0.85 1]);
    set (h, 'edgecolor','none');

    hold on

h = plot (delta/(2*pi),y*r2d);
    set(h,'linewidth',2)
    xlabel ('\delta_x / 2\pi')
    set (gca,'ylim',[-90,90]);
    set (gca,'xlim', [x1 x2])
    set (gca,'layer','top')
    set (gca,'ytick',[-80:20:80])
    
    set (gca,'tickdir','out');
    set (gca,'ticklength',[0.015 0.025]);
    title (sprintf('Beam direction as function of phasing angle when Dx=%1.1f',D));
    ylabel ('\theta_0 (\circ)')

    hold on

    for x = -1.5:1:1.5
    
        xx = x/D;
        
        if abs (xx) < 1
            y = asin (xx) * r2d;
            plot ( [x x], [-90 y], 'b--');
            plot ( [x1 x], [y y], 'b--');
        end
    
    end

    hold off

%=====================================================================
% (B) Then draw the beam directions separately for the 2*nmax+1
% "grating" directions, while restricting delta to [-pi,pi]
%
% The full range of theta is covered in both cases, but perhaps
% the representation (B) is to be preferred as it shows more
% clearly the directional ambiguity.
%=====================================================================

getfigure ('beamsteering (2)',[0.25 2.5 7 4], 500);

    % mark out the "base zone" by painting a colored band

    y1 = asin (-0.5/D) * r2d;
    y2 = asin (0.5/D) * r2d;
    h = area ( [x1 x2], [y2 y2], y1 );
        set (h, 'facecolor', [0.85 0.85 1]);
        set (h, 'edgecolor','none');

        hold on

    nmax = ceil(D);

    for n = -nmax:1:nmax
    
        xx1 = max ([n-0.5, -D]); 
        xx2 = min ([n+0.5, D] );
    
        x = linspace (xx1,xx2,1000);
        xx = x(2:end-1);

        K = abs(xx/D) <= 1;
        y = asin (xx(K)/D);
    
        h = plot ( xx(K)-n, y(K)*r2d);
        set (h, 'linewidth', 2)
    end
    
    
    hold off
    
    
    set (gca,'layer','top')
    set (gca,'ytick',[-80:20:80])
    set (gca,'ylim',[-90,90]);
    xlabel ('\delta_x / 2\pi')
    ylabel ('\theta_0 (\circ)')
    set (gca,'xlim', [x1 x2])
    set (gca,'tickdir','out');
    set (gca,'ticklength',[0.015 0.025]);
    title (sprintf('Beam direction as function of phasing angle when Dx=%1.1f',D));
    ylabel ('\theta_0 (\circ)')

%end beamsteering

