function [X,Y,Z] = grating_dir ( Dx, Dy, deltaX, deltaY)

% [X,Y,Z] = grating_dir ( Dx, Dy, deltaX, deltaY)
%
% Find all grating lobe directions with kz >= 0. We return the
% unit vectors giving the lobe directions.
%
% The first element [X(1),Y(1),Z(1)] gives the main beam direction.
%
% We solve the direction of the unit vector
%
%    u = [x, y, z]  = (kx,ky,kz)/||k||
%
% from
%
%   x = kx/k = nx/Dx + deltaX/(2pi Dx)
%   y = ky/k = ny/Dy + deltaY/(2pi Dy)
%
% where n1 and n2 must be such that
%
%   x^2 + y^2 <= 1
%
% so that we get z from  
%
%   z = +sqrt( 1 - x^2 + y^2);
%


%
% 14-Oct-2005 Jm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Find the "bounding box" for (nx, ny) by requiring that
% 
%   -1 <= x <= 1 and   -1 <= y <= 1

Nx = ceil (-deltaX/(2*pi) - Dx) : 1 : floor (-deltaX/(2*pi) + Dx);
Ny = ceil (-deltaY/(2*pi) - Dy) : 1 : floor (-deltaY/(2*pi) + Dy);

% Then go trough all (nx,ny) in the box

m = 0;
X = [];
Y = [];
Z = [];
x0 = NaN;
for ny = Ny(end:-1:1)
    for nx = Nx
        x = (nx*2*pi + deltaX)/(2*pi*Dx);
        y = (ny*2*pi + deltaY)/(2*pi*Dy);
        z2 = 1 - x^2 - y^2;
        if z2 >= 0
            z = sqrt (z2);
            if nx == 0 && ny == 0
                x0 = x;
                y0 = y;
                z0 = z;
            else
                m = m+1;
                X(m) = x;
                Y(m) = y;
                Z(m) = z;
            end
        end
    end
end
if ~isnan(x0)
    X = [x0 X];
    Y = [y0 Y];
    Z = [z0 Z];
end
%grating_angles
    
