function D = plot_maxgain (TH)

% D = PLOT_MAXGAIN (TH)
%
% Plot the array directivity as function of beam zenith angle.
%
% Optionally, if a vector of beam polar angles (TH, radians) is given, 
% return the corresponding values of directivities D. The points (TH,D)
% are also marked in the directivity plot then.
%
% The array element's parameters are hard-coded in the m-file source
% code below. 
%
% See also 
%   simplegain.m, check_array_element.m, powerint.m

% 21-Sep-2006 Jm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

d2r = pi/180;
r2d = 180/pi;
Ewidth = []; Egain = [];

% ==================== BEGIN INPUT =============================


Mx = 12;              % Number of elements in x direction.
My = 4;              % Number of elements in y direction.  

Dx = 1.4;            % Element distance in x-direction devided by wavelenth.
Dy = 1.4;            % Element distance in y-direction devided by wavelenth.

%%          Specify either the element beamwidth or the power gain   

%Ewidth = 40*d2r;     % Beam width of an individual "yagi", radians.
                      % We assume power gain Gy of the form
                      % Gy = Gy0*[cos(theta/2)]^ny. The factors ny and Gy0
                      % are computed below, to give the required BW.
                      % (The beamwidth refers to the 0.5*Gy0 beamwidth, not
                      % to the 3dB beamwidth.) 
Egain = 10^(11.5/10); % Power gain as multiplicative factor (not in dB)  

Ephi =00*d2r;         % Azimuth of the element
Eth = 35*d2r;         % Zenith angle of the element

%           Polar angle grid

th0  = (-10:0.1:90)*d2r;    % for gain
th02 = (-10:0.5:80)*d2r;    % for beamwidth
xlim = [-20 100];
xtick = -10:10:90;

% =================== END INPUT =================================

% The vertical plain 

phi0 = Ephi;

ED.phi   = Ephi;
ED.th    = Eth;
ED.width = Ewidth;
ED.gain  = Egain;
ED = check_array_element (ED);

AD.Mx = Mx;
AD.My = My;
AD.Dx = Dx;
AD.Dy = Dy;

for k = 1:length(th0)
    % Element gain

    GE(k) = simplegain ( phi0, th0(k), ED);

    % Antenna directivity

    [P,DI(k)] = powerint ( AD, ED, phi0, th0(k));
end

if nargin > 0
    D = zeros(size(TH));
    for k = 1:length(TH)
        [dummy,D(k)] = powerint ( AD, ED, phi0, TH(k));
    end
else
    TH = [];
end

% Vertical beamwidth

for k = 1:length(th02)
    W(k) = beamwidth ( Dx, Dy, Mx, My, phi0, th02(k), 0.5);
end

GE_dB = 10*log10(GE);

% Reference gain = (Number of elements) x (element directivity)

Gref = AD.Mx*AD.My*ED.gain;
Gref_dB = 10*log10(Gref);

% Maximum of antenna directivity

[DImax,k] = max(DI);   
Th_max = th0(k);
DImax_dB = 10*log10(DImax);

% Antenna directivity normalized by the reference gain

Di = DI/Gref;
Di_dB = 10*log10(Di);
Dimax_dB = 10*log10(DImax/Gref);

% Regime where gain is at most 3 dB down from _reference_ gain.

K3dB = find ( Di_dB > -3 );
Ka = min(K3dB);
Kb = max(K3dB);
Th_a = th0(Ka);
Th_b = th0(Kb);
K3 = find ( th02 > Th_a & th02 < Th_b );

% Directivity DI_EB in the elements' broadside direction

[PE,DI_E] = powerint ( AD, ED, ED.phi, ED.th);
DI_E_dB = 10*log10(DI_E);
DiE_dB = 10*log10(DI_E/Gref);

% Vertical beamwidht at 10 degr and 90 degr elevation

W0 = beamwidth ( Dx, Dy, Mx, My, phi0, 0*d2r, 0.5);

%==============================================================
%       GRAPHS
%==============================================================

getfigure('plot_maxgain',[0.25 2.5 6 7], 500);

% (1) Header

h1 = subplot('position',[0.02,0.78,0.885,0.10]);

    fmt1 = 'ARRAY       Mx=%d  My=%d';
      fmt2 = '  Dx=%1.2f  Dy=%1.2f';
    hdr{1} = sprintf ([fmt1 fmt2], ...
                Mx, My, ...
                  Dx, Dy);

    fmt1 = 'ELEMENT   \\phi=%1.0f\\circ \\theta=%1.0f\\circ';  
      fmt2 = '  W=%1.0f\\circ';
        fmt3 = '  Ge=%1.1f dBi';
    hdr{2} = sprintf ([fmt1 fmt2 fmt3], ...
                            ED.phi*r2d, ED.th*r2d, ...
                                ED.width*r2d, ...
                                    10*log10 (ED.gain) );

    fmt1 = 'BEAM        W(0\\circ)=%1.2f\\circ';
        fmt2 = ' Gref=%1.1f dBi  D(%1.1f\\circ)=%1.1f  D(%1.1f\\circ)=%1.1f';
    hdr{3} = sprintf ( [fmt1 fmt2], ...
                W0*r2d, ...
                    Gref_dB, Th_max*r2d, DImax_dB, ...
                                    ED.th*r2d,DI_E_dB );

figureheader(h1, hdr);

% (2) First panel: element gain and antenna's normalized directivity

subplot('position',[0.15,0.45,0.7550,0.28]);
h = plot ( th0*r2d, GE_dB, 'c--', th0*r2d, Di_dB, 'c-', ...
            th0(K3dB)*r2d, Di_dB(K3dB), 'b-' );
%            Th_max*r2d, Dimax_dB,'r*',...
%            ED.th*r2d, DiE_dB,'ro');

    set (h, 'linewidth', 1.5);
    set (h(1),'color', [ 0.2 0.7 0.2]);

    if ~isempty(TH)
        hold on
        for k = 1:length (TH)
            Dity = 10*log10(D(k)/Gref);
            h = plot ( TH(k)*r2d, Dity,'bo'); set(h,'linewidth',1.5);
        end
        hold off
    end

    set (gca, 'ylim', [-10, ceil(DImax_dB)-Gref_dB+1]);
    set (gca, 'xlim',[xlim]);
    set (gca, 'xtick',xtick);
    xlabel ('Zenith angle (\circ)');
    ylabel ('D / G_{ref} (dB)')   
    setax

% (3) Second panel: vertical beam width

subplot('position',[0.15,0.15,0.7550,0.20]);
h = plot ( th02*r2d, W*r2d,'c-', th02(K3)*r2d, W(K3)*r2d,'b-');
    set (h, 'linewidth', 1.5);
    set (gca, 'xlim',[xlim]);
    set (gca, 'xtick',xtick);
    xlabel ('Zenith angle (\circ)');
    ylabel ('Vertical width (\circ)')
    setax

%end plot_maxgain


%==============================================================
%       SUBFUNCTIONS
%==============================================================

function setax
%-------------
set (gca,'layer','top');
set (gca,'tickdir','out');
set (gca,'ticklength',[0.015 0.035]);


